'
'
' ####################
' #####  PROLOG  #####
' ####################
'
' Draw a shape using path gradients.
'
PROGRAM	"star"
VERSION	"0.0001"
'
	IMPORT	"xst"   		' standard library	: required by most programs
	IMPORT  "xsx"				' extended std library
'	IMPORT	"xio"				' console io library
	IMPORT	"gdi32"     ' gdi32.dll
	IMPORT  "user32"    ' user32.dll
	IMPORT  "kernel32"  ' kernel32.dll
	IMPORT  "comctl32"	' comctl32.dll			: common controls library
	IMPORT  "gdiplus"

DECLARE FUNCTION Entry            ()
DECLARE FUNCTION WndProc          (hwnd, msg, wParam, lParam)
DECLARE FUNCTION InitGui          ()
DECLARE FUNCTION RegisterWinClass (className$, addrWndProc, icon$, menu$)
DECLARE FUNCTION CreateWindows    ()
DECLARE FUNCTION NewWindow        (className$, titleBar$, style, x, y, w, h, exStyle)
DECLARE FUNCTION MessageLoop      ()
DECLARE FUNCTION CleanUp          ()
DECLARE FUNCTION OnPaint          (hDC)
DECLARE FUNCTION ARGB             (UBYTE alpha, UBYTE r, UBYTE g, UBYTE b)
DECLARE FUNCTION gdipLoadImage    (file$)
'
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	STATIC	entry
'
	IF entry THEN RETURN							' enter once
	entry =  $$TRUE										' enter occured

'	XioCreateConsole (title$, 500)		' create console, if console is not wanted, comment out this line
	InitGui ()												' initialize program and libraries
	IF CreateWindows () THEN QUIT (0)	' create windows and other child controls
	MessageLoop ()										' the main message loop
	CleanUp ()												' unregister all window classes
'	XioFreeConsole ()									' free console

END FUNCTION
'
'
' ########################
' #####  WndProc ()  #####
' ########################
'
FUNCTION WndProc (hWnd, msg, wParam, lParam)

	PAINTSTRUCT ps

	SELECT CASE msg

		CASE $$WM_DESTROY :
			PostQuitMessage(0)

		CASE $$WM_PAINT :
			hdc = BeginPaint (hWnd, &ps)
			OnPaint (hdc)
			EndPaint (hWnd, &ps)

		CASE ELSE :
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT

END FUNCTION
'
'
' ########################
' #####  InitGui ()  #####
' ########################
'
FUNCTION InitGui ()

	SHARED hInst
	SHARED gdiPlusToken
	GdiplusStartupInput gsi
	
	hInst = GetModuleHandleA (0) 	' get current instance handle
	IFZ hInst THEN QUIT (0)
	InitCommonControls ()					' initialize comctl32.dll library
	
	' initialize gdiplus
	gsi.GdiplusVersion = 1
	err = GdiplusStartup (&gdiPlusToken, &gsi, NULL) 
	IF err THEN
		err$ = "GdiplusStartup error: " + STRING(err)
		MessageBoxA (0, &err$, &"Gdiplus error", $$MB_OK | $$MB_ICONWARNING)
		QUIT (0)
	END IF 	  

END FUNCTION
'
'
' #################################
' #####  RegisterWinClass ()  #####
' #################################
'
FUNCTION RegisterWinClass (className$, addrWndProc, icon$, menu$)

	SHARED hInst
	WNDCLASS wc

	wc.style           = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc     = addrWndProc
	wc.cbClsExtra      = 0
	wc.cbWndExtra      = 0
	wc.hInstance       = hInst
	wc.hIcon           = LoadIconA (hInst, &icon$)
	wc.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wc.hbrBackground   = GetStockObject ($$WHITE_BRUSH) ' $$COLOR_BTNFACE + 1 
	wc.lpszMenuName    = &menu$
	wc.lpszClassName   = &className$

	IFZ RegisterClassA (&wc) THEN RETURN ($$TRUE)

END FUNCTION
'
'
' ##############################
' #####  CreateWindows ()  #####
' ##############################
'
FUNCTION CreateWindows ()

' register window class
	className$  = "GdiPlusGradientDemo"
	addrWndProc = &WndProc()
	icon$ 			= "scrabble"
	menu$ 			= ""
	IF RegisterWinClass (@className$, addrWndProc, @icon$, @menu$) THEN RETURN ($$TRUE)

' create main window
	titleBar$ = "Gdiplus Gradient Demo"
	style 		= $$WS_OVERLAPPEDWINDOW
	w 				= 380
	h 				= 400
	exStyle		= 0
	#winMain = NewWindow (@className$, @titleBar$, style, x, y, w, h, exStyle)
	IFZ #winMain THEN RETURN ($$TRUE)

	XstCenterWindow (#winMain)							' center window position
	ShowWindow (#winMain, $$SW_SHOWNORMAL)	' show window

END FUNCTION
'
'
' ##########################
' #####  NewWindow ()  #####
' ##########################
'
FUNCTION NewWindow (className$, titleBar$, style, x, y, w, h, exStyle)

	SHARED hInst

	RETURN CreateWindowExA (exStyle, &className$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)

END FUNCTION

'
' ############################
' #####  MessageLoop ()  #####
' ############################
'
FUNCTION MessageLoop ()

	MSG msg

' main message loop

	IF LIBRARY(0) THEN RETURN								' main program executes message loop

	DO																			' the message loop
		ret = GetMessageA (&msg, NULL, 0, 0)	' retrieve next message from queue

		SELECT CASE ret
			CASE  0 : RETURN msg.wParam					' WM_QUIT message
			CASE -1 : RETURN $$TRUE							' error
			CASE ELSE:
				hwnd = GetActiveWindow ()
				IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &msg)) THEN		' send only non-dialog messages
  				TranslateMessage (&msg)					' translate virtual-key messages into character messages
  				DispatchMessageA (&msg)					' send message to window callback function WndProc()
				END IF
		END SELECT
	LOOP

END FUNCTION
'
'
' ########################
' #####  CleanUp ()  #####
' ########################
'
FUNCTION CleanUp ()

	SHARED hInst, className$
	SHARED gdiPlusToken

	UnregisterClassA (&className$, hInst)
	GdiplusShutdown (gdiplusToken)

END FUNCTION
'
' #####################
' #####  OnPaint  #####
' #####################
'
' Paint window
'
FUNCTION OnPaint (hDC)

	POINT pt[9]

	pt[0].x = 150
	pt[0].y = 0
	pt[1].x = 200
	pt[1].y = 100
	pt[2].x = 300
	pt[2].y = 100
	pt[3].x = 224
	pt[3].y = 150
	pt[4].x = 300
	pt[4].y = 300
	pt[5].x = 150
	pt[5].y = 200
	pt[6].x = 0
	pt[6].y = 300
	pt[7].x = 75
	pt[7].y = 150
	pt[8].x = 0
	pt[8].y = 100
	pt[9].x = 100
	pt[9].y = 100
	
	GdipCreateFromHDC (hDC, &graphics)
	
	' Create a GraphicsPath object
	GdipCreatePath ($$FillModeAlternate, &path)
	
	' Use the array of points to construct a path.
	GdipAddPathLine2I (path, &pt[], 10)
	
	' Use the path to construct a path gradient brush.
	GdipCreatePathGradientFromPath (path, &brush)
	
	' Set the color at the center of the path to red.
	GdipSetPathGradientCenterColor (brush, ARGB(255, 255, 0, 0))
	
	' Set the colors of the points in the array.
	Count = 10
	DIM Colors[Count-1]
	
	Colors[0] = ARGB (255, 255, 0, 0)
	Colors[1] = ARGB (255, 255, 255, 0)
	Colors[2] = ARGB (255, 0, 255, 0)
	Colors[3] = ARGB (255, 255, 255, 0)
	Colors[4] = ARGB (255, 0, 255, 255)
	Colors[5] = ARGB (255, 255, 255, 0)
	Colors[6] = ARGB (255, 0, 255, 0)
	Colors[7] = ARGB (255, 255, 255, 0)
	Colors[8] = ARGB (255, 0, 255, 255)
	Colors[9] = ARGB (255, 255, 255, 0)
	GdipSetPathGradientSurroundColorsWithCount(brush, &Colors[], &Count)
	
	' Fill the path with the path gradient brush.
	GdipFillPath (graphics, brush, path)
	
	' Cleanup
	GdipDeletePath (path)
	GdipDeleteBrush (brush)
	GdipDeleteGraphics (graphics)


END FUNCTION
'
' ##################
' #####  ARGB  #####
' ##################
'
' Return 32-bit color value form argb values
'
FUNCTION ARGB (UBYTE alpha, UBYTE r, UBYTE g, UBYTE b)

	RETURN alpha << 24 + r << 16 + g << 8 + b 

END FUNCTION
'
' ###########################
' #####  gdipLoadImage  #####
' ###########################
'
' Load image file into bitmap format
' Return value is handle to bitmap image object
'
FUNCTION gdipLoadImage (file$)

	IFZ file$ THEN RETURN
  fileWSize = (LEN (file$) * 2) + 2
  lpFileW = GlobalAlloc ($$GMEM_FIXED, fileWSize)
  MultiByteToWideChar (0, 0, &file$, -1, lpFileW, fileWSize)
  GdipCreateBitmapFromFile (lpFileW, &bitmap)
  GlobalFree (lpFileW)
  IF bitmap THEN RETURN bitmap

END FUNCTION
END PROGRAM
